# pylint: disable=e0401, c0302
# -*- coding: utf-8 -*-
"""handler for xca ca handler"""

from __future__ import print_function
import os
import sqlite3
import uuid
import json
import datetime
from typing import List, Tuple, Dict
from cryptography import x509
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization, hashes
from cryptography.x509 import (
    BasicConstraints,
    ExtendedKeyUsage,
    SubjectKeyIdentifier,
    AuthorityKeyIdentifier,
    KeyUsage,
    SubjectAlternativeName,
)
from cryptography.x509.oid import ExtendedKeyUsageOID
from OpenSSL import crypto as pyossslcrypto
from acme2certifier.acme_srv.helper import (
    b64_decode,
    b64_encode,
    b64_url_recode,
    build_pem_file,
    cert_serial_get,
    config_eab_profile_load,
    config_enroll_config_log_load,
    config_headerinfo_load,
    config_profile_load,
    convert_byte_to_string,
    convert_string_to_byte,
    csr_cn_get,
    csr_san_get,
    eab_profile_header_info_check,
    eab_profile_revocation_check,
    enrollment_config_log,
    error_dic_get,
    load_config,
    uts_now,
    uts_to_date_utc,
)

# Define constants
DEFAULT_DATE_FORMAT = "%Y%m%d%H%M%SZ"
COLUMN_NOT_IN_TABLE_MSG = "column: %s not in %s table"


def dict_from_row(row):
    """small helper to convert the output of a "select" command into a dictionary"""
    return dict(zip(row.keys(), row))


class CAhandler(object):
    """CA  handler"""

    def __init__(self, debug: bool = False, logger: object = None):
        self.debug = debug
        self.logger = logger
        self.xdb_file = None
        self.xdb_permission = "660"
        self.passphrase = None
        self.issuing_ca_name = None
        self.issuing_ca_key = None
        self.cert_validity_days = 365
        self.ca_cert_chain_list = []
        self.template_name = None
        self.header_info_field = None
        self.eab_handler = None
        self.eab_profiling = False
        self.enrollment_config_log = False
        self.enrollment_config_log_skip_list = []
        self.profiles = {}
        self.profile_mapping_field = "template_name"

    def __enter__(self):
        """Makes ACMEHandler a Context Manager"""
        if not self.xdb_file:
            self._config_load()
        return self

    def __exit__(self, *args):
        """cose the connection at the end of the context"""

    def _asn1_stream_parse(self, asn1_stream: str = None) -> Dict[str, str]:
        """parse asn_string"""

        self.logger.debug("CAhandler._asn1_stream_parse()")
        oid_dic = {
            "2.5.4.3": "commonName",
            "2.5.4.4": "surname",
            "2.5.4.5": "serialNumber",
            "2.5.4.6": "countryName",
            "2.5.4.7": "localityName",
            "2.5.4.8": "stateOrProvinceName",
            "2.5.4.9": "streetAddress",
            "2.5.4.10": "organizationName",
            "2.5.4.11": "organizationalUnitName",
            "2.5.4.12": "title",
            "2.5.4.13": "description",
            "2.5.4.42": "givenName",
        }

        dn_dic = {}
        if asn1_stream:

            # cut first 8 bytes which are bogus
            # asn1_stream = asn1_stream[8:]

            # split stream
            stream_list = asn1_stream.split(b"\x06\x03\x55")
            # we have to remove the first element from list as it contains junk
            stream_list.pop(0)

            for ele in stream_list:
                oid = f"2.5.{ele[0]}.{ele[1]}"
                if oid in oid_dic:
                    value_len = ele[3]
                    value = ele[4 : 4 + value_len]
                    dn_dic[oid_dic[oid]] = value.decode("utf-8")

            self.logger.debug("CAhandler._asn1_stream_parse() ended: %s", bool(dn_dic))
        return dn_dic

    def _ca_cert_load(self) -> Tuple[object, int]:
        """load ca key from database"""
        self.logger.debug("CAhandler._ca_cert_load({%s)", self.issuing_ca_name)

        # query database for key
        self._db_open()
        pre_statement = """SELECT * from view_certs WHERE name LIKE ?"""
        self.cursor.execute(pre_statement, [self.issuing_ca_name])
        try:
            db_result = dict_from_row(self.cursor.fetchone())
        except Exception:
            self.logger.error(
                "Certificate lookup in database failed: %s", self.cursor.fetchone()
            )
            db_result = {}
        self._db_close()

        ca_cert = None
        ca_id = None

        if "cert" in db_result:
            try:
                ca_cert = x509.load_der_x509_certificate(
                    b64_decode(self.logger, db_result["cert"]),
                    backend=default_backend(),
                )
                ca_id = db_result["id"]
            except Exception as err_:
                self.logger.error(
                    "Failed to load CA certificate from database: %s", err_
                )

        return (ca_cert, ca_id)

    def _ca_key_load(self) -> object:
        """load ca key from database"""
        self.logger.debug("CAhandler._ca_key_load(%s)", self.issuing_ca_key)

        # query database for key
        self._db_open()
        pre_statement = """SELECT * from view_private WHERE name LIKE ?"""
        self.cursor.execute(pre_statement, [self.issuing_ca_key])
        try:
            db_result = dict_from_row(self.cursor.fetchone())
        except Exception as err:
            self.logger.error("Failed to load CA private key from database: %s", err)
            db_result = {}
        self._db_close()

        ca_key = None
        if db_result and "private" in db_result:
            try:
                private_key = f'-----BEGIN ENCRYPTED PRIVATE KEY-----\n{db_result["private"]}\n-----END ENCRYPTED PRIVATE KEY-----'
                ca_key = serialization.load_pem_private_key(
                    convert_string_to_byte(private_key),
                    password=convert_string_to_byte(self.passphrase),
                    backend=default_backend(),
                )
            except Exception as err_:
                self.logger.error(
                    "Failed to load CA private key from database: %s", err_
                )
        else:
            self.logger.error("Failed to load CA private key: %s", db_result)

        self.logger.debug("CAhandler._ca_key_load() ended")
        return ca_key

    def _ca_load(self) -> Tuple[object, object, int]:
        """load ca key and cert"""
        self.logger.debug("CAhandler._ca_load()")
        ca_key = self._ca_key_load()
        ca_cert, ca_id = self._ca_cert_load()

        self.logger.debug("CAhandler._ca_load() ended")
        return (ca_key, ca_cert, ca_id)

    def _cdp_list_generate(self, cdp_string: str = None) -> List[str]:
        """generate cdp list"""
        self.logger.debug("CAhandler._cdp_list_generate()")

        cdp_list = []
        if cdp_string:
            for ele in cdp_string.split(","):
                cdp_list.append(
                    x509.DistributionPoint(
                        [x509.UniformResourceIdentifier(ele.strip())],
                        crl_issuer=None,
                        reasons=None,
                        relative_name=None,
                    )
                )

        self.logger.debug("CAhandler._cdp_list_generate() ended")
        return cdp_list

    def _cert_insert(self, cert_dic: Dict[str, str] = None) -> int:
        """insert new entry to request table"""
        self.logger.debug("CAhandler._cert_insert()")

        row_id = None
        if cert_dic:
            if all(
                key in cert_dic
                for key in (
                    "item",
                    "serial",
                    "issuer",
                    "ca",
                    "cert",
                    "iss_hash",
                    "hash",
                )
            ):
                # pylint: disable=R0916
                if (
                    isinstance(cert_dic["item"], int)
                    and isinstance(cert_dic["issuer"], int)
                    and isinstance(cert_dic["ca"], int)
                    and isinstance(cert_dic["iss_hash"], int)
                    and isinstance(cert_dic["iss_hash"], int)
                    and isinstance(cert_dic["hash"], int)
                ):
                    self._db_open()
                    self.cursor.execute(
                        """INSERT INTO CERTS(item, serial, issuer, ca, cert, hash, iss_hash) VALUES(:item, :serial, :issuer, :ca, :cert, :hash, :iss_hash)""",
                        cert_dic,
                    )
                    row_id = self.cursor.lastrowid
                    self._db_close()
                else:
                    self.logger.error(
                        "Certificate insert aborted due to wrong datatypes: %s",
                        cert_dic,
                    )
            else:
                self.logger.error(
                    "Certificate insert aborted due to incomplete dataset: %s", cert_dic
                )
        else:
            self.logger.error("Certificate insert aborted: dataset is empty")

        self.logger.debug("CAhandler._cert_insert() ended with row_id: %s", row_id)
        return row_id

    def _cert_search(self, column: str, value: str) -> Dict[str, str]:
        """load ca key from database"""
        self.logger.debug("CAhandler._cert_search({%s:%s)", column, value)

        if not self._identifier_check("items", column):
            self.logger.warning(COLUMN_NOT_IN_TABLE_MSG, column, "items")
            return {}

        # query database for key
        self._db_open()
        pre_statement = f"""SELECT * from items WHERE type == 3 and {column} LIKE ?"""
        self.cursor.execute(pre_statement, [value])

        cert_result = {}
        try:
            item_result = dict_from_row(self.cursor.fetchone())
        except Exception:
            self.logger.error(
                "Certificate item search in database failed: %s", self.cursor.fetchone()
            )
            item_result = {}

        if item_result:
            item_id = item_result["id"]
            pre_statement = """SELECT * from certs WHERE item LIKE ?"""
            self.cursor.execute(pre_statement, [item_id])
            try:
                cert_result = dict_from_row(self.cursor.fetchone())
            except Exception:
                self.logger.error(
                    "Certificate search in database failed for item: %s", item_id
                )

        self._db_close()
        self.logger.debug("CAhandler._cert_search() ended")
        return cert_result

    def _cert_subject_generate(
        self, req: object, request_name: str, dn_dic: Dict[str, str] = None
    ) -> str:
        """set subject"""
        self.logger.debug("CAhandler._cert_subject_generate()")

        if not bool(req.subject):
            self.logger.info("Rewrite CN to %s", request_name)
            subject = x509.Name(
                [x509.NameAttribute(x509.NameOID.COMMON_NAME, request_name)]
            )
        else:
            subject = req.subject

        if dn_dic:
            # modify subject according to template
            subject = self._subject_modify(subject, dn_dic)

        self.logger.debug("CAhandler._cert_subject_generate() ended")
        return subject

    def _cert_sign(
        self, csr: str, request_name: str, ca_key: object, ca_cert: object, ca_id: int
    ) -> Tuple[str, str]:  # pylint: disable=R0913
        self.logger.debug("Certificate._cert_sign()")

        if self.enrollment_config_log:
            self.enrollment_config_log_skip_list.extend(["dbs", "passphrase", "cursor"])
            enrollment_config_log(
                self.logger, self, self.enrollment_config_log_skip_list
            )

        # load template if configured
        if self.template_name:
            dn_dic, template_dic = self._template_load()
        else:
            dn_dic = {}
            template_dic = {}

        # creating a rest from CSR
        req = x509.load_pem_x509_csr(convert_string_to_byte(csr), default_backend())

        # set cert_validity
        if "validity" in template_dic:
            self.logger.debug(
                "Take validity from template: %s", template_dic["validity"]
            )
            # take validity from template
            cert_validity = template_dic["validity"]
        else:
            cert_validity = self.cert_validity_days

        # create object for certificate
        builder = x509.CertificateBuilder()

        # set not valid before
        builder = builder.not_valid_before(datetime.datetime.now(datetime.timezone.utc))
        builder = builder.not_valid_after(
            datetime.datetime.now(datetime.timezone.utc)
            + datetime.timedelta(days=cert_validity)
        )
        builder = builder.issuer_name(ca_cert.subject)
        builder = builder.serial_number(uuid.uuid4().int & (1 << 63) - 1)
        builder = builder.public_key(req.public_key())

        # get extension list from CSR
        csr_extensions_list = req.extensions
        extension_list = self._extension_list_generate(
            template_dic, req, ca_cert, csr_extensions_list
        )

        # add extensions (copy from CSR and take the ones we constructed)
        for extension in extension_list:
            builder = builder.add_extension(
                extension["name"], critical=extension["critical"]
            )

        # get subject and set to builder
        builder = builder.subject_name(
            self._cert_subject_generate(req, request_name, dn_dic)
        )

        # sign certificate
        cert = builder.sign(
            private_key=ca_key, algorithm=hashes.SHA256(), backend=default_backend()
        )

        # get serial
        serial = cert.serial_number
        # get hsshes
        issuer_subject_hash = self._subject_name_hash_get(ca_cert)
        cert_subject_hash = self._subject_name_hash_get(cert)

        # store certificate
        self._store_cert(
            ca_id,
            request_name,
            f"{serial:X}",
            convert_byte_to_string(
                b64_encode(self.logger, cert.public_bytes(serialization.Encoding.DER))
            ),
            cert_subject_hash,
            issuer_subject_hash,
        )

        cert_bundle = self._pemcertchain_generate(
            convert_byte_to_string(cert.public_bytes(serialization.Encoding.PEM)),
            convert_byte_to_string(ca_cert.public_bytes(serialization.Encoding.PEM)),
        )
        cert_raw = convert_byte_to_string(
            b64_encode(self.logger, cert.public_bytes(serialization.Encoding.DER))
        )

        self.logger.debug("Certificate._cert_sign() ended.")
        return (cert_bundle, cert_raw)

    def _columnnames_get(self, table: str) -> List[str]:
        """get columns of a table"""
        self.logger.debug("CAhandler.columns_get(%s)", table)

        self._db_open()
        pre_statement = f"SELECT * from {table}"
        self.cursor.execute(pre_statement)
        result = [column[0] for column in self.cursor.description]
        self._db_close()

        self.logger.debug(
            "CAhandler.columns_get() ended with: %s elements", len(result)
        )
        return result

    def _config_check(self) -> str:
        """check config for consitency"""
        self.logger.debug("CAhandler._config_check()")
        error = None

        if self.xdb_file:
            if not os.path.exists(self.xdb_file):
                error = f"xdb_file {self.xdb_file} does not exist"
                self.xdb_file = None
        else:
            error = "xdb_file must be specified in config file"

        if not error and not self.issuing_ca_name:
            error = "issuing_ca_name must be set in config file"

        if error:
            self.logger.debug("CAhandler config error: %s", error)

        if not self.issuing_ca_key:
            self.logger.debug(
                "use self.issuing_ca_name as self.issuing_ca_key: %s",
                self.issuing_ca_name,
            )
            self.issuing_ca_key = self.issuing_ca_name

        self.logger.debug("CAhandler._config_check() ended")
        return error

    def _config_load(self):
        """ " load config from file"""
        self.logger.debug("CAhandler._config_load()")
        config_dic = load_config(self.logger, "CAhandler")

        if "CAhandler" in config_dic:
            self.xdb_file = config_dic.get(
                "CAhandler", "xdb_file", fallback=self.xdb_file
            )
            self.xdb_permission = config_dic.get(
                "CAhandler", "xdb_permission", fallback=self.xdb_permission
            )
            self.issuing_ca_name = config_dic.get(
                "CAhandler", "issuing_ca_name", fallback=self.issuing_ca_name
            )
            self.issuing_ca_key = config_dic.get(
                "CAhandler", "issuing_ca_key", fallback=self.issuing_ca_key
            )
            self.template_name = config_dic.get(
                "CAhandler", self.profile_mapping_field, fallback=self.template_name
            )

        if "passphrase_variable" in config_dic["CAhandler"]:
            try:
                self.passphrase = os.environ[
                    config_dic.get("CAhandler", "passphrase_variable")
                ]
            except Exception as err:
                self.logger.error(
                    "Could not load passphrase_variable:%s",
                    err,
                )

        if "passphrase" in config_dic["CAhandler"]:
            # overwrite passphrase specified in variable
            if self.passphrase:
                self.logger.info("Overwrite passphrase_variable")
            self.passphrase = config_dic.get("CAhandler", "passphrase")

        if "ca_cert_chain_list" in config_dic["CAhandler"]:
            try:
                self.ca_cert_chain_list = json.loads(
                    config_dic.get("CAhandler", "ca_cert_chain_list")
                )
            except Exception:
                self.logger.error('Parameter "ca_cert_chain_list" cannot be loaded')

        # load profiling
        self.eab_profiling, self.eab_handler = config_eab_profile_load(
            self.logger, config_dic
        )

        # load profiles
        self.profiles = config_profile_load(self.logger, config_dic)

        # load header info
        self.header_info_field = config_headerinfo_load(self.logger, config_dic)

        # load enrollment config log
        (
            self.enrollment_config_log,
            self.enrollment_config_log_skip_list,
        ) = config_enroll_config_log_load(self.logger, config_dic)

    def _csr_import(self, csr, request_name):
        """check existance of csr and load into db"""
        self.logger.debug("CAhandler._csr_import()")

        csr_info = self._csr_search("request", csr)

        if not csr_info:

            # csr does not exist in db - lets import it
            insert_date = uts_to_date_utc(uts_now(), DEFAULT_DATE_FORMAT)
            item_dic = {
                "type": 2,
                "comment": "from acme2certifier",
                "source": 2,
                "date": insert_date,
                "name": request_name,
            }
            row_id = self._item_insert(item_dic)

            # insert csr
            csr_info = {"item": row_id, "signed": 1, "request": csr}
            self._csr_insert(csr_info)

        self.logger.debug("CAhandler._csr_import() ended")
        return csr_info

    def _csr_insert(self, csr_dic: Dict[str, str] = None) -> int:
        """insert new entry to request table"""
        self.logger.debug("CAhandler._csr_insert()")
        row_id = None
        if csr_dic:
            if all(key in csr_dic for key in ("item", "signed", "request")):
                # item and signed must be integer
                if isinstance(csr_dic["item"], int) and isinstance(
                    csr_dic["signed"], int
                ):
                    self._db_open()
                    self.cursor.execute(
                        """INSERT INTO REQUESTS(item, signed, request) VALUES(:item, :signed, :request)""",
                        csr_dic,
                    )
                    row_id = self.cursor.lastrowid
                    self._db_close()
                else:
                    self.logger.error(
                        "CSR insert aborted due to wrong datatypes: %s", csr_dic
                    )
            else:
                self.logger.error(
                    "CSR insert aborted due to incomplete dataset: %s", csr_dic
                )
        else:
            self.logger.error("CSR insert aborted: dataset is empty")

        self.logger.debug("CAhandler._csr_insert() ended with row_id: %s", row_id)
        return row_id

    def _csr_search(self, column: str, value: str) -> Dict[str, str]:
        """load ca key from database"""
        self.logger.debug("CAhandler._csr_search()")

        if not self._identifier_check("view_requests", column):
            self.logger.warning(COLUMN_NOT_IN_TABLE_MSG, column, "view_requests")
            return {}

        # query database for key
        self._db_open()
        pre_statement = f"""SELECT * from view_requests WHERE {column} LIKE ?"""
        self.cursor.execute(pre_statement, [value])

        try:
            db_result = dict_from_row(self.cursor.fetchone())
        except Exception:
            db_result = {}
        self._db_close()
        self.logger.debug("CAhandler._csr_search() ended with: %s", bool(db_result))
        return db_result

    def _db_check(self):
        """do various checks on database"""
        self.logger.debug("CAhandler._db_check()")
        error = None

        st = os.stat(self.xdb_file)
        oct_perm = oct(st.st_mode)[-3:]

        # test open failure
        if not os.access(self.xdb_file, os.R_OK):
            error = f"xdb_file {self.xdb_file} is not readable"
        elif not os.access(self.xdb_file, os.W_OK):
            error = f"xdb_file {self.xdb_file} is not writeable"
        # warns if permissions are to wide
        elif (
            int(oct_perm[0]) > int(self.xdb_permission[0])
            or int(oct_perm[1]) > int(self.xdb_permission[1])
            or int(oct_perm[2]) > int(self.xdb_permission[2])
        ):
            self.logger.warning(
                "File permissions %s for '%s' are too permissive. Should be %s.",
                oct_perm,
                self.xdb_file,
                self.xdb_permission,
            )

        # validates passphrase against database
        if not error:
            ca_key = self._ca_key_load()
            if not ca_key:
                error = "ca_key_load failed. PLease check passphrase"

        self.logger.debug("CAhandler._db_check() ended with: %s", error)
        return error

    def _db_open(self):
        """opens db and sets cursor"""
        # pylint: disable=W0201
        self.dbs = sqlite3.connect(self.xdb_file)
        self.dbs.row_factory = sqlite3.Row
        # pylint: disable=W0201
        self.cursor = self.dbs.cursor()

    def _db_close(self):
        """commit and close"""
        # self.logger.debug('DBStore._db_close()')
        self.dbs.commit()
        self.dbs.close()
        # self.logger.debug('DBStore._db_close() ended')

    def _extended_keyusage_generate(
        self, template_dic: Dict[str, str], _csr_extensions_dic: Dict[str, str] = None
    ) -> Tuple[bool, List[str]]:
        """set generate extended key usage extenstion"""
        self.logger.debug("CAhandler._extended_keyusage_generate()")

        eku_list = []
        if "eKeyUse" in template_dic:
            # eku included in tempalate
            eku_mapping_dic = {
                "clientAuth": ExtendedKeyUsageOID.CLIENT_AUTH,
                "serverAuth": ExtendedKeyUsageOID.SERVER_AUTH,
                "codeSigning": ExtendedKeyUsageOID.CODE_SIGNING,
                "emailProtection": ExtendedKeyUsageOID.EMAIL_PROTECTION,
                "timeStamping": ExtendedKeyUsageOID.TIME_STAMPING,
                "OCSPSigning": ExtendedKeyUsageOID.OCSP_SIGNING,
                "eKeyUse": "eKeyUse",  # this is just for testing
            }
            # backwards compatibility with cryptography module coming Ubuntu 22.04
            if hasattr(ExtendedKeyUsageOID, "KERBEROS_PKINIT_KDC"):
                eku_mapping_dic["pkInitKDC"] = ExtendedKeyUsageOID.KERBEROS_PKINIT_KDC

            if "ekuCritical" in template_dic:
                try:
                    ekuc = bool(int(template_dic["ekuCritical"]))
                except Exception:
                    self.logger.error(
                        "Failed to convert EKU critical flag to int, defaulting to False"
                    )
                    ekuc = False
            else:
                ekuc = False

            for ele in template_dic["eKeyUse"].split(", "):
                if ele in eku_mapping_dic:
                    eku_list.append(eku_mapping_dic[ele])

        else:
            # neither extension nor template
            eku_list = None
            ekuc = False

        return (ekuc, eku_list)

    def _extension_list_default(self, ca_cert: str = None, cert: str = None):
        """set default extension list"""
        self.logger.debug("CAhandler._extension_list_default()")

        extension_list = [
            {"name": BasicConstraints(ca=False, path_length=None), "critical": True},
            {
                "name": KeyUsage(
                    digital_signature=True,
                    key_encipherment=True,
                    content_commitment=False,
                    data_encipherment=False,
                    key_agreement=False,
                    key_cert_sign=False,
                    crl_sign=False,
                    encipher_only=False,
                    decipher_only=False,
                ),
                "critical": True,
            },
            {
                "name": ExtendedKeyUsage([ExtendedKeyUsageOID.SERVER_AUTH]),
                "critical": False,
            },
        ]
        if cert:
            extension_list.append(
                {
                    "name": SubjectKeyIdentifier.from_public_key(cert.public_key()),
                    "critical": False,
                },
            )
        if ca_cert:
            extension_list.append(
                {
                    "name": AuthorityKeyIdentifier.from_issuer_public_key(
                        ca_cert.public_key()
                    ),
                    "critical": False,
                }
            )

        self.logger.debug("CAhandler._extension_list_default() ended")
        return extension_list

    def _extension_list_generate(
        self,
        template_dic: Dict[str, str],
        cert: str,
        ca_cert: str,
        csr_extensions_list: List[str] = None,
    ) -> List[str]:
        """set extension list"""
        self.logger.debug("CAhandler._extension_list_generate()")

        csr_extensions_dic = {}
        if csr_extensions_list:
            for ext in csr_extensions_list:
                csr_extensions_dic[convert_byte_to_string(ext.oid._name)] = (
                    ext  # pylint: disable=W0212
                )

        if template_dic:
            # prcoess xca template
            extension_list = self._xca_template_process(
                template_dic, csr_extensions_dic, cert, ca_cert
            )
        else:
            extension_list = self._extension_list_default(ca_cert, cert)

        # add subjectAltName(s)
        if "subjectAltName" in csr_extensions_dic:
            # pylint: disable=C2801
            self.logger.debug(
                "CAhandler._extension_list_generate(): adding subAltNames: %s",
                csr_extensions_dic["subjectAltName"].__str__(),
            )
            extension_list.append(
                {
                    "name": SubjectAlternativeName(
                        csr_extensions_dic["subjectAltName"].value
                    ),
                    "critical": False,
                }
            )

        self.logger.debug("CAhandler._extension_list_generate() ended")
        return extension_list

    def _identifier_check(self, table: str, identifier: str) -> bool:
        """check if identifier is in table"""
        self.logger.debug("CAhandler._identifier_check(%s, %s)", identifier, table)
        if "." in identifier:
            # we have a table.column name
            table, identifier = identifier.split(".", 1)
            self.logger.debug(
                "CAhandler._identifier_check(): modified table/identifier to %s/%s",
                table,
                identifier,
            )
        elif "__" in identifier:
            # we have a table__column name
            table, identifier = identifier.split("__", 1)
            self.logger.debug(
                "CAhandler._identifier_check(): modified table/identifier to %s/%s",
                table,
                identifier,
            )
        if self._table_check(table):
            columnname_list = self._columnnames_get(table)
            result = True if identifier in columnname_list else False
        else:
            self.logger.warning("Table '%s' does not exist in the database.", table)
            result = False

        self.logger.debug("CAhandler._identifier_check() ended with: %s", result)
        return result

    def _item_insert(self, item_dic: Dict[str, str] = None) -> int:
        """insert new entry to item_table"""
        self.logger.debug("CAhandler._item_insert()")
        row_id = None
        # insert
        if item_dic:
            if all(
                key in item_dic for key in ("name", "type", "source", "date", "comment")
            ):
                if isinstance(item_dic["type"], int) and isinstance(
                    item_dic["source"], int
                ):
                    self._db_open()
                    self.cursor.execute(
                        """INSERT INTO ITEMS(name, type, source, date, comment) VALUES(:name, :type, :source, :date, :comment)""",
                        item_dic,
                    )
                    row_id = self.cursor.lastrowid
                    # update stamp field
                    data_dic = {"stamp": row_id}
                    self.cursor.execute(
                        """UPDATE ITEMS SET stamp = :stamp WHERE id = :stamp""",
                        data_dic,
                    )
                    self._db_close()
                else:
                    self.logger.error(
                        "Item insert aborted due to wrong datatypes: %s", item_dic
                    )
            else:
                self.logger.error(
                    "Item insert aborted due to incomplete dataset: %s", item_dic
                )
        else:
            self.logger.error("Item insert aborted: dataset is empty")

        self.logger.debug("CAhandler._item_insert() ended with row_id: %s", row_id)
        return row_id

    def _keyusage_generate(
        self, template_dic: Dict[str, str], _csr_extensions_dic: Dict[str, str] = None
    ) -> Tuple[bool, Dict[str, str]]:
        """set generate key usage extenstion"""
        self.logger.debug("CAhandler._keyusage_generate()")

        if "keyUse" in template_dic:
            if "kuCritical" in template_dic:
                try:
                    kuc = bool(int(template_dic["kuCritical"]))
                except Exception:
                    kuc = False
            else:
                kuc = False
            kup = template_dic["keyUse"]
        else:
            kuc = False
            kup = 0

        # generate key-usage extension
        ku_dic = self._kue_generate(kup)

        return (kuc, ku_dic)

    def _kue_generate(self, kuval: int = 0, ku_csr: str = None) -> Dict[str, str]:
        """set generate key usage extension"""
        self.logger.debug("CAhandler._kue_generate()")

        # convert keyusage value from template
        if kuval:
            try:
                kuval = int(kuval)
            except Exception:
                self.logger.error(
                    "Keyusage value conversion to int failed, defaulting to 0"
                )
                kuval = 0

        if kuval:
            # we have a key-usage value from template
            self.logger.debug("Generate KeyUsage Extension with data from template")
            ku_dic = self._ku_dict_generate(kuval)
        elif ku_csr:
            # no data from template but data from csr
            self.logger.debug("Generate KeyUsage Extension with data from csr")
            ku_dic = ku_csr
        else:
            # no data from template no data from csr - default (23)
            self.logger.debug("Generate KeyUsage Extension with value 23")
            ku_dic = self._ku_dict_generate(23)

        self.logger.debug("CAhandler._kue_generate() ended with: %s", ku_dic)
        return ku_dic

    def _ku_dict_generate(self, kuval: int = 0) -> Dict[str, str]:
        self.logger.debug("CAhandler._ku_dict_generate(%s)", kuval)

        # generate and reverse key_usage_list
        key_usage_list = [
            "digital_signature",
            "content_commitment",
            "key_encipherment",
            "data_encipherment",
            "key_agreement",
            "key_cert_sign",
            "crl_sign",
            "encipher_only",
            "decipher_only",
        ]
        key_usage_dic = {
            "digital_signature": False,
            "content_commitment": False,
            "key_encipherment": False,
            "data_encipherment": False,
            "key_agreement": False,
            "key_cert_sign": False,
            "crl_sign": False,
            "encipher_only": False,
            "decipher_only": False,
        }

        kubin = f"{kuval:b}"[::-1]
        for idx, ele in enumerate(kubin):
            if ele == "1":
                key_usage_dic[key_usage_list[idx]] = True

        self.logger.debug("CAhandler._ku_dict_generate() ended with: %s", key_usage_dic)
        return key_usage_dic

    def _pemcertchain_generate(self, ee_cert: str, issuer_cert: str = None) -> str:
        """build pem chain"""
        self.logger.debug("CAhandler._pemcertchain_generate()")

        if issuer_cert:
            pem_chain = f"{ee_cert}{issuer_cert}"
        else:
            pem_chain = ee_cert

        for cert in self.ca_cert_chain_list:
            cert_dic = self._cert_search("items.name", cert)
            if cert_dic and "cert" in cert_dic:
                ca_cert = x509.load_der_x509_certificate(
                    b64_decode(self.logger, cert_dic["cert"]), backend=default_backend()
                )
                pem_chain = f"{pem_chain}{convert_byte_to_string(ca_cert.public_bytes(serialization.Encoding.PEM))}"

        self.logger.debug("CAhandler._pemcertchain_generate() ended")
        return pem_chain

    def _requestname_get(self, csr: str = None) -> str:
        """get request name"""
        self.logger.debug("CAhandler._requestname_get()")

        # try to get cn for a name in database
        request_name = csr_cn_get(self.logger, csr)
        if not request_name:
            san_list = csr_san_get(self.logger, csr)
            try:
                (
                    _identifiier,
                    request_name,
                ) = san_list[
                    0
                ].split(":")
            except Exception:
                self.logger.error(
                    "Failed to split SAN from CSR subjectAltName: %s", san_list
                )

        self.logger.debug("CAhandler._requestname_get() ended with: %s", request_name)
        return request_name

    def _revocation_insert(self, rev_dic: Dict[str, str] = None) -> int:
        """insert new entry to into revocation_table"""
        self.logger.debug("CAhandler._revocation_insert()")
        row_id = None
        # insert
        if rev_dic:
            if all(
                key in rev_dic
                for key in ("caID", "serial", "date", "invaldate", "reasonBit")
            ):
                if isinstance(rev_dic["caID"], int) and isinstance(
                    rev_dic["reasonBit"], int
                ):
                    self._db_open()
                    self.cursor.execute(
                        """INSERT INTO REVOCATIONS(caID, serial, date, invaldate, reasonBit) VALUES(:caID, :serial, :date, :invaldate, :reasonBit)""",
                        rev_dic,
                    )
                    row_id = self.cursor.lastrowid
                    self._db_close()
                else:
                    self.logger.error(
                        "Revocation insert aborted due to wrong datatypes: %s", rev_dic
                    )
            else:
                self.logger.error(
                    "Revocation insert aborted due to incomplete dataset: %s", rev_dic
                )
        else:
            self.logger.error("Revocation insert aborted: dataset is empty")

        self.logger.debug(
            "CAhandler._revocation_insert() ended with row_id: %s", row_id
        )
        return row_id

    def _revocation_check(
        self, serial: str, ca_id: int, err_msg_dic: Dict[str, str] = None
    ) -> Tuple[int, str, str]:
        self.logger.debug("CAhandler.revoke(%s/%s)", serial, ca_id)

        # check if certificate has alreay been revoked:
        if not self._revocation_search("serial", serial):
            rev_dic = {
                "caID": ca_id,
                "serial": serial,
                "date": uts_to_date_utc(uts_now(), DEFAULT_DATE_FORMAT),
                "invaldate": uts_to_date_utc(uts_now(), DEFAULT_DATE_FORMAT),
                "reasonBit": 0,
            }
            row_id = self._revocation_insert(rev_dic)
            if row_id:
                code = 200
                message = None
                detail = None
            else:
                code = 500
                message = err_msg_dic["serverinternal"]
                detail = "database update failed"
        else:
            code = 400
            message = err_msg_dic["alreadyrevoked"]
            detail = "Certificate has already been revoked"

        self.logger.debug("CAhandler.revoke() ended with: %s", code)
        return (code, message, detail)

    def _revocation_search(self, column: str, value: str) -> Dict[str, str]:
        """load ca key from database"""
        self.logger.debug("CAhandler._revocation_search()")

        if not self._identifier_check("revocations", column):
            self.logger.warning(COLUMN_NOT_IN_TABLE_MSG, column, "revocations")
            return {}
        # query database for key
        self._db_open()
        pre_statement = f"""SELECT * from revocations WHERE {column} LIKE ?"""
        self.cursor.execute(pre_statement, [value])

        try:
            db_result = dict_from_row(self.cursor.fetchone())
        except Exception:
            db_result = {}
        self._db_close()
        self.logger.debug("CAhandler._revocation_search() ended")
        return db_result

    # pylint: disable=R0913
    def _store_cert(
        self,
        ca_id: int,
        cert_name: str,
        serial: str,
        cert: str,
        name_hash: str,
        issuer_hash: str,
    ) -> int:
        """store certificate to database"""
        self.logger.debug("CAhandler._store_cert()")

        # insert certificate into item table
        insert_date = uts_to_date_utc(uts_now(), DEFAULT_DATE_FORMAT)
        item_dic = {
            "type": 3,
            "comment": "from acme2certifier",
            "source": 2,
            "date": insert_date,
            "name": cert_name,
        }
        row_id = self._item_insert(item_dic)
        # insert certificate to cert table
        cert_dic = {
            "item": row_id,
            "serial": serial,
            "issuer": ca_id,
            "ca": 0,
            "cert": cert,
            "iss_hash": issuer_hash,
            "hash": name_hash,
        }
        _row_id = self._cert_insert(cert_dic)  # lgtm [py/unused-local-variable]

        self.logger.debug("CAhandler._store_cert() ended")

    def _stream_split(self, byte_stream: str = None) -> Tuple[str, str]:
        """split template in asn1 structure and utf_stream"""
        self.logger.debug("CAhandler._stream_split()")
        asn1_stream = None
        utf_stream = None

        # convert to byte if not already done
        byte_stream = convert_string_to_byte(byte_stream)

        if byte_stream:
            # search pattern
            pos = byte_stream.find(b"\x00\x00\x00\x0c") + 4
            if pos != 3:
                # split file 3 bcs find returns -1 in case of no-match
                asn1_stream = byte_stream[:pos]
                utf_stream = byte_stream[pos:]

        self.logger.debug(
            "CAhandler._stream_split() ended: %s:%s",
            bool(asn1_stream),
            bool(utf_stream),
        )
        return (asn1_stream, utf_stream)

    def _stub_func(self, parameter: str) -> str:
        """ " load config from file"""
        self.logger.debug("CAhandler._stub_func(%s)", parameter)
        self.logger.debug("CAhandler._stub_func() ended")
        return parameter

    def _subject_name_hash_get(self, cert: str = None) -> int:
        """get subject name hash"""
        self.logger.debug("CAhandler._subject_name_hash_get()")

        pyopenssl_cert = pyossslcrypto.X509.from_cryptography(cert)
        pyopenssl_subject_name_hash = pyopenssl_cert.subject_name_hash() & 0x7FFFFFFF

        return pyopenssl_subject_name_hash

    def _subject_modify(self, subject: str, dn_dic: Dict[str, str] = None) -> str:
        """modify subject name"""
        self.logger.debug("CAhandler._subject_modify()")

        subject_name_list = []

        if "organizationalUnitName" in dn_dic and dn_dic["organizationalUnitName"]:
            self.logger.info("Rewrite OU to %s", dn_dic["organizationalUnitName"])
            subject_name_list.append(
                x509.NameAttribute(
                    x509.NameOID.ORGANIZATIONAL_UNIT_NAME,
                    dn_dic["organizationalUnitName"],
                )
            )
        if "organizationName" in dn_dic and dn_dic["organizationName"]:
            self.logger.info("Rewrite O to %s", dn_dic["organizationName"])
            subject_name_list.append(
                x509.NameAttribute(
                    x509.NameOID.ORGANIZATION_NAME, dn_dic["organizationName"]
                )
            )
        if "localityName" in dn_dic and dn_dic["localityName"]:
            self.logger.info("Rewrite L to %s", dn_dic["localityName"])
            subject_name_list.append(
                x509.NameAttribute(x509.NameOID.LOCALITY_NAME, dn_dic["localityName"])
            )
        if "stateOrProvinceName" in dn_dic and dn_dic["stateOrProvinceName"]:
            self.logger.info("Rewrite ST to %s", dn_dic["stateOrProvinceName"])
            subject_name_list.append(
                x509.NameAttribute(
                    x509.NameOID.STATE_OR_PROVINCE_NAME, dn_dic["stateOrProvinceName"]
                )
            )
        if "countryName" in dn_dic and dn_dic["countryName"]:
            self.logger.info("Rewrite C to %s", dn_dic["countryName"])
            subject_name_list.append(
                x509.NameAttribute(x509.NameOID.COUNTRY_NAME, dn_dic["countryName"])
            )

        if subject_name_list:
            subject = x509.Name([*subject, *subject_name_list])

        self.logger.debug("CAhandler._subject_modify() ended")
        return subject

    def _table_check(self, table: str) -> bool:
        """get all tables in db"""
        self.logger.debug("DBStore.tables_get()")
        self._db_open()
        pre_statement = (
            "SELECT name FROM sqlite_master WHERE type='table' or type == 'view'"
        )
        self.cursor.execute(pre_statement)
        tables_list = [row[0] for row in self.cursor.fetchall()]
        self._db_close()
        result = True if table in tables_list else False
        self.logger.debug("DBStore._table_check() ended with: %s", result)
        return result

    def _template_load(self) -> Tuple[Dict[str, str], Dict[str, str]]:
        """load template from database"""
        self.logger.debug("CAhandler._template_load(%s)", self.template_name)
        # query database for template
        self._db_open()
        pre_statement = """SELECT * from view_templates WHERE name LIKE ?"""
        self.cursor.execute(pre_statement, [self.template_name])
        try:
            db_result = dict_from_row(self.cursor.fetchone())
        except Exception:
            self.logger.error("template lookup failed: %s", self.cursor.fetchone())
            db_result = {}

        # parse template
        dn_dic = {}
        template_dic = {}
        if "template" in db_result:
            byte_stream = b64_decode(self.logger, db_result["template"])
            dn_dic, template_dic = self._template_parse(byte_stream)
        self._db_close()
        self.logger.debug("CAhandler._template_load() ended")

        return (dn_dic, template_dic)

    def _template_parse(
        self, byte_string: str = None
    ) -> Tuple[Dict[str, str], Dict[str, str]]:
        """process template"""
        self.logger.debug("CAhandler._template_parse()")
        asn1_stream, utf_stream = self._stream_split(byte_string)

        dn_dic = {}
        if asn1_stream:
            dn_dic = self._asn1_stream_parse(asn1_stream)

        template_dic = {}
        if utf_stream:
            template_dic = self._utf_stream_parse(utf_stream)
            if template_dic:
                # replace '' with None
                template_dic = {
                    k: None if not v else v for k, v in template_dic.items()
                }

                template_dic["validity"] = self._validity_calculate(template_dic)

        self.logger.debug("CAhandler._template_parse() ended")
        return (dn_dic, template_dic)

    def _utf_stream_parse(self, utf_stream: str = None) -> Dict[str, str]:
        """parse template information from utf_stream into dictitionary"""
        self.logger.debug("CAhandler._utf_stream_parse()")
        template_dic = {}

        if utf_stream:
            stream_list = utf_stream.split(b"\x00\x00\x00")

            # iterate list and clean up parameter
            parameter_list = []
            for idx, ele in enumerate(stream_list):
                ele = ele.replace(b"\x00", b"")
                if idx > 0:
                    # strip the first character
                    ele = ele[1:]
                if ele == b"eKeyUse\xff\xff\xff\xff":
                    self.logger.info(
                        "Hack to skip template with empty eku - maybe a bug in xca..."
                    )
                else:
                    parameter_list.append(ele.decode("utf-8"))

            if parameter_list:
                if len(parameter_list) % 2 != 0:
                    # remove last element from list if amount of list entries is uneven
                    parameter_list.pop()
                # convert list into a directory
                template_dic = {
                    item: parameter_list[index + 1]
                    for index, item in enumerate(parameter_list)
                    if index % 2 == 0
                }

        self.logger.debug("CAhandler._utf_stream_parse() ended: %s", bool(template_dic))
        return template_dic

    def _validity_calculate(self, template_dic: Dict[str, str] = None) -> int:
        """calculate validity in days"""
        self.logger.debug("CAhandler._validity_calculate()")

        cert_validity = 365
        if "validM" in template_dic and "validN" in template_dic:
            if template_dic["validM"] == "0":
                # validity in days
                cert_validity = int(template_dic["validN"])
            elif template_dic["validM"] == "1":
                # validity in months
                cert_validity = int(template_dic["validN"]) * 30
            elif template_dic["validM"] == "2":
                # validity in months
                cert_validity = int(template_dic["validN"]) * 365
        else:
            cert_validity = 365

        self.logger.debug(
            "CAhandler._validity_calculate() ended with: %s", cert_validity
        )
        return cert_validity

    def _xca_template_process(
        self,
        template_dic: Dict[str, str],
        csr_extensions_dic: Dict[str, str],
        cert: str,
        ca_cert: str,
    ) -> List[str]:
        """add xca template"""
        self.logger.debug("Certificate._xca_template_process()")

        extension_list = [
            {
                "name": SubjectKeyIdentifier.from_public_key(cert.public_key()),
                "critical": False,
            },
            {
                "name": AuthorityKeyIdentifier.from_issuer_public_key(
                    ca_cert.public_key()
                ),
                "critical": False,
            },
        ]

        # key_usage
        kuc, ku_dic = self._keyusage_generate(template_dic, csr_extensions_dic)
        extension_list.append({"name": KeyUsage(**ku_dic), "critical": kuc})

        # extended key_usage
        ekuc, eku_list = self._extended_keyusage_generate(
            template_dic, csr_extensions_dic
        )
        if eku_list:
            extension_list.append(
                {"name": ExtendedKeyUsage(eku_list), "critical": ekuc}
            )

        # add cdp
        if "crlDist" in template_dic and template_dic["crlDist"]:
            cdp_list = self._cdp_list_generate(template_dic["crlDist"])
            extension_list.append(
                {"name": x509.CRLDistributionPoints(cdp_list), "critical": False}
            )

        # add basicConstraints
        if "ca" in template_dic:
            if "bcCritical" in template_dic:
                try:
                    bcc = bool(int(template_dic["bcCritical"]))
                except Exception:
                    bcc = False
            else:
                bcc = False

            if template_dic["ca"] == "1":
                extension_list.append(
                    {
                        "name": BasicConstraints(ca=True, path_length=None),
                        "critical": bcc,
                    }
                )
            elif template_dic["ca"] == "2":
                extension_list.append(
                    {
                        "name": BasicConstraints(ca=False, path_length=None),
                        "critical": bcc,
                    }
                )

        return extension_list

    def handler_check(self):
        """check if handler is ready"""
        self.logger.debug("CAhandler.check()")

        error = self._config_check()
        if not error:
            error = self._db_check()

        self.logger.debug("CAhandler.check() ended with %s", error)
        return error

    def enroll(self, csr: str = None) -> Tuple[str, str, str, str]:
        """enroll certificate"""
        # pylint: disable=R0914, R0915
        self.logger.debug("CAhandler.enroll()")

        cert_bundle = None
        cert_raw = None
        error = self._config_check()

        if not error:
            error = self._db_check()

        # fmt: off
        if not error:
            error = eab_profile_header_info_check(self.logger, self, csr, self.profile_mapping_field)
        # fmt: on

        if not error:
            request_name = self._requestname_get(csr)

            if request_name:
                # import CSR to database
                _csr_info = self._csr_import(
                    csr, request_name
                )  # lgtm [py/unused-local-variable]

                # prepare the CSR to be signed
                csr = build_pem_file(
                    self.logger, None, b64_url_recode(self.logger, csr), None, True
                )

                # load ca cert and key
                ca_key, ca_cert, ca_id = self._ca_load()

                if ca_key and ca_cert and ca_id:
                    cert_bundle, cert_raw = self._cert_sign(
                        csr, request_name, ca_key, ca_cert, ca_id
                    )
                else:
                    error = "ca lookup failed"
            else:
                error = "request_name lookup failed"
        self.logger.debug("Certificate.enroll() ended")
        return (error, cert_bundle, cert_raw, None)

    def poll(
        self, cert_name: str, poll_identifier: str, _csr: str
    ) -> Tuple[str, str, str, str, bool]:
        """poll status of pending CSR and download certificates"""
        self.logger.debug("CAhandler.poll()")

        error = "Method not implemented."
        cert_bundle = None
        cert_raw = None
        rejected = False
        self._stub_func(cert_name)

        self.logger.debug("CAhandler.poll() ended")
        return (error, cert_bundle, cert_raw, poll_identifier, rejected)

    def revoke(
        self, cert: str, _rev_reason: str = "unspecified", _rev_date: str = None
    ) -> Tuple[int, str, str]:
        """revoke certificate"""
        self.logger.debug("CAhandler.revoke()")

        err_msg_dic = error_dic_get(self.logger)

        # modify handler configuration in case of eab profiling
        if self.eab_profiling:
            eab_profile_revocation_check(self.logger, self, cert)

        if self.xdb_file:
            # load ca cert and key
            _ca_key, _ca_cert, ca_id = self._ca_load()

            serial = cert_serial_get(self.logger, cert)
            if serial:
                serial = f"{serial:X}"

            if ca_id and serial:
                code, message, detail = self._revocation_check(
                    serial, ca_id, err_msg_dic
                )
            else:
                code = 500
                message = err_msg_dic["serverinternal"]
                detail = "certificate lookup failed"
        else:
            code = 500
            message = err_msg_dic["serverinternal"]
            detail = "configuration error"

        self.logger.debug("Certificate.revoke() ended")
        return (code, message, detail)

    def trigger(self, payload: str) -> Tuple[str, str, str]:
        """process trigger message and return certificate"""
        self.logger.debug("CAhandler.trigger()")

        error = "Method not implemented."
        cert_bundle = None
        cert_raw = None
        self._stub_func(payload)

        self.logger.debug("CAhandler.trigger() ended with error: %s", error)
        return (error, cert_bundle, cert_raw)



# -*- coding: utf-8 -*-
"""Unit tests for DatabaseCertificateRepository abstraction over DBstore"""

import os
import unittest
from unittest.mock import MagicMock, patch
import sys

# Add the parent directory to sys.path so we can import acme_srv
sys.path.insert(0, ".")
sys.path.insert(1, "..")


class TestCertificateRepository(unittest.TestCase):
    def setUp(self):
        models_mock = MagicMock()
        models_mock.DBstore.return_value = (
            MagicMock()
        )
        modules = {"acme2certifier.acme_srv.db_handler": models_mock}
        patch.dict("sys.modules", modules).start()
        from acme2certifier.acme_srv.certificate_repository import DatabaseCertificateRepository

        self.logger = MagicMock()
        self.db = MagicMock()
        self.repo = DatabaseCertificateRepository(self.db, self.logger)

    # --- search_certificates ---
    def test_001_search_no_vlist_success(self):
        self.db.certificates_search.return_value = [{"name": "c1"}]
        res = self.repo.search_certificates("name", "c1")
        self.db.certificates_search.assert_called_once_with("name", "c1")
        self.assertEqual(res, [{"name": "c1"}])

    def test_002_search_with_vlist_success(self):
        self.db.certificates_search.return_value = []
        res = self.repo.search_certificates("name", "c1", ["name", "csr"])
        self.db.certificates_search.assert_called_once_with(
            "name", "c1", ["name", "csr"]
        )
        self.assertEqual(res, [])

    def test_003_search_exception_returns_none(self):
        self.db.certificates_search.side_effect = RuntimeError("db")
        res = self.repo.search_certificates("name", "c1")
        self.assertIsNone(res)
        self.logger.critical.assert_called()

    # --- get_certificate_info ---
    def test_004_get_certificate_info_success(self):
        self.db.certificate_lookup.return_value = {"name": "c1", "cert": "pem"}
        res = self.repo.get_certificate_info("c1")
        self.db.certificate_lookup.assert_called_once()
        self.assertEqual(res["name"], "c1")

    def test_005_get_certificate_info_exception_returns_empty(self):
        self.db.certificate_lookup.side_effect = RuntimeError("db")
        res = self.repo.get_certificate_info("c1")
        self.assertEqual(res, {})

    def test_006_get_certificate_info_none_from_db_passes_through(self):
        self.db.certificate_lookup.side_effect = None
        self.db.certificate_lookup.return_value = None
        res = self.repo.get_certificate_info("c1")
        self.assertIsNone(res)

    # --- add / update / delete (name variant) ---
    def test_007_add_certificate_success(self):
        self.db.certificate_add.return_value = True
        ok = self.repo.add_certificate({"name": "c1"})
        self.assertTrue(ok)

    def test_008_add_certificate_exception_returns_false(self):
        self.db.certificate_add.side_effect = RuntimeError("db")
        ok = self.repo.add_certificate({"name": "c1"})
        self.assertFalse(ok)

    def test_009_delete_certificate_success(self):
        self.db.certificate_delete.return_value = True
        ok = self.repo.delete_certificate("c1")
        self.assertTrue(ok)

    def test_010_delete_certificate_exception_returns_false(self):
        self.db.certificate_delete.side_effect = RuntimeError("db")
        ok = self.repo.delete_certificate("c1")
        self.assertFalse(ok)

    # --- account check / update order (middle methods) ---
    def test_011_get_account_check_result_success(self):
        self.db.certificate_account_check.return_value = {"ok": True}
        res = self.repo.get_account_check_result("acc", "cert")
        self.assertEqual(res, {"ok": True})

    def test_012_get_account_check_result_exception_returns_none(self):
        self.db.certificate_account_check.side_effect = RuntimeError("db")
        res = self.repo.get_account_check_result("acc", "cert")
        self.assertIsNone(res)

    def test_013_update_order_success_true(self):
        self.db.order_update.return_value = (
            None  # method has no return, repo returns True
        )
        ok = self.repo.update_order({"name": "o1", "status": "valid"})
        self.assertTrue(ok)

    def test_014_update_order_exception_returns_false(self):
        self.db.order_update.side_effect = RuntimeError("db")
        ok = self.repo.update_order({"name": "o1", "status": "valid"})
        self.assertFalse(ok)

    # --- get_orders_by_account ---
    def test_015_get_orders_by_account_success_list(self):
        self.db.orders_search.return_value = [{"name": "o1"}]
        res = self.repo.get_orders_by_account("acc")
        self.assertEqual(res, [{"name": "o1"}])

    def test_016_get_orders_by_account_empty_to_list(self):
        self.db.orders_search.return_value = None
        res = self.repo.get_orders_by_account("acc")
        self.assertEqual(res, [])

    def test_017_get_orders_by_account_exception_returns_empty(self):
        self.db.orders_search.side_effect = RuntimeError("db")
        res = self.repo.get_orders_by_account("acc")
        self.assertEqual(res, [])

    # --- get_certificate_by_order ---
    def test_018_get_certificate_by_order_success(self):
        self.db.certificate_lookup.return_value = {"name": "c1"}
        res = self.repo.get_certificate_by_order("o1")
        self.assertEqual(res, {"name": "c1"})

    def test_019_get_certificate_by_order_exception_returns_empty(self):
        self.db.certificate_lookup.side_effect = RuntimeError("db")
        res = self.repo.get_certificate_by_order("o1")
        self.assertEqual(res, {})

    # --- store_certificate_operation_log ---
    def test_020_store_certificate_operation_log_success(self):
        self.db.cahandler_add.return_value = True
        ok = self.repo.store_certificate_operation_log("c1", "store", "success")
        self.assertTrue(ok)
        self.db.cahandler_add.assert_called_once()

    def test_021_store_certificate_operation_log_exception_returns_false(self):
        self.db.cahandler_add.side_effect = RuntimeError("db")
        ok = self.repo.store_certificate_operation_log("c1", "store", "success")
        self.assertFalse(ok)

    # --- bottom API (compatibility) ---
    def test_022_certificate_account_check_success(self):
        self.db.certificate_account_check.return_value = True
        res = self.repo.certificate_account_check("acc", "cert")
        self.assertTrue(res)

    def test_023_certificate_account_check_exception_returns_none(self):
        self.db.certificate_account_check.side_effect = RuntimeError("db")
        res = self.repo.certificate_account_check("acc", "cert")
        self.assertIsNone(res)

    def test_024_certificate_lookup_with_vlist_success(self):
        self.db.certificate_lookup.return_value = {"name": "c1"}
        res = self.repo.certificate_lookup("name", "c1", ["name"])
        self.db.certificate_lookup.assert_called_once_with("name", "c1", ["name"])
        self.assertEqual(res, {"name": "c1"})

    def test_025_certificate_lookup_without_vlist_success(self):
        self.db.certificate_lookup.reset_mock()
        self.db.certificate_lookup.return_value = {"name": "c2"}
        res = self.repo.certificate_lookup("name", "c2")
        self.db.certificate_lookup.assert_called_once_with("name", "c2")
        self.assertEqual(res, {"name": "c2"})

    def test_026_certificate_lookup_exception_returns_empty(self):
        self.db.certificate_lookup.side_effect = RuntimeError("db")
        res = self.repo.certificate_lookup("name", "c1")
        self.assertEqual(res, {})

    def test_027_certificate_add_success_returns_id(self):
        self.db.certificate_add.return_value = 123
        res = self.repo.certificate_add({"name": "c1"})
        self.assertEqual(res, 123)

    def test_028_certificate_add_exception_returns_none(self):
        self.db.certificate_add.side_effect = RuntimeError("db")
        res = self.repo.certificate_add({"name": "c1"})
        self.assertIsNone(res)

    def test_029_certificate_delete_success(self):
        self.db.certificate_delete.return_value = True
        ok = self.repo.certificate_delete("name", "c1")
        self.assertTrue(ok)

    def test_030_certificate_delete_exception_returns_false(self):
        self.db.certificate_delete.side_effect = RuntimeError("db")
        ok = self.repo.certificate_delete("name", "c1")
        self.assertFalse(ok)

    def test_031_order_lookup_with_vlist_success(self):
        self.db.order_lookup.return_value = {"name": "o1"}
        res = self.repo.order_lookup("name", "o1", ["name"])
        self.db.order_lookup.assert_called_once_with("name", "o1", ["name"])
        self.assertEqual(res, {"name": "o1"})

    def test_032_order_lookup_without_vlist_success(self):
        self.db.order_lookup.reset_mock()
        self.db.order_lookup.return_value = {"name": "o2"}
        res = self.repo.order_lookup("name", "o2")
        self.db.order_lookup.assert_called_once_with("name", "o2")
        self.assertEqual(res, {"name": "o2"})

    def test_033_order_lookup_exception_returns_empty(self):
        self.db.order_lookup.side_effect = RuntimeError("db")
        res = self.repo.order_lookup("name", "o1")
        self.assertEqual(res, {})

    def test_034_order_update_success(self):
        self.db.order_update.return_value = True
        ok = self.repo.order_update({"name": "o1"})
        self.assertTrue(ok)

    def test_035_order_update_exception_returns_false(self):
        self.db.order_update.side_effect = RuntimeError("db")
        ok = self.repo.order_update({"name": "o1"})
        self.assertFalse(ok)

    def test_036_search_expired_certificates_returns_results(self):
        # Simulate dbstore returning a list of expired certificates
        certs = [
            {"name": "expired1", "expire_uts": 100, "cert": "pem1"},
            {"name": "expired2", "expire_uts": 200, "cert": "pem2"},
        ]
        self.db.certificates_search.return_value = certs
        result = self.repo.search_expired_certificates(
            123456, ["name", "expire_uts", "cert"]
        )
        self.db.certificates_search.assert_called_once_with(
            "expire_uts", 123456, ["name", "expire_uts", "cert"], "<="
        )
        self.assertEqual(result, certs)

    def test_037_search_expired_certificates_returns_empty(self):
        # Simulate dbstore returning an empty list
        self.db.certificates_search.return_value = []
        result = self.repo.search_expired_certificates(
            123456, ["name", "expire_uts", "cert"]
        )
        self.db.certificates_search.assert_called_once()
        self.assertEqual(result, [])

    def test_038_search_expired_certificates_raises_exception(self):
        # Simulate dbstore raising an exception
        self.db.certificates_search.side_effect = RuntimeError("db error")
        result = self.repo.search_expired_certificates(
            123456, ["name", "expire_uts", "cert"]
        )
        self.assertEqual(result, [])
        self.logger.critical.assert_called()


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Error class"""

# pylint: disable=c0209
from __future__ import print_function


class Error(object):
    """error messages"""

    def __init__(self, debug=None, logger=None):
        self.debug = debug
        self.logger = logger

    def _acme_errormessage(self, message):
        """dictionary containing the implemented acme error messages"""
        self.logger.debug("Error.acme_errormessage({0})".format(message))
        error_dic = {
            "urn:ietf:params:acme:error:accountDoesNotExist": None,
            "urn:ietf:params:acme:error:badCSR": None,
            "urn:ietf:params:acme:error:badNonce": "JWS has invalid anti-replay nonce",
            "urn:ietf:params:acme:error:invalidContact": "The provided contact URI was invalid",
            "urn:ietf:params:acme:error:malformed": None,
            "urn:ietf:params:acme:error:serverInternal": None,
            "urn:ietf:params:acme:error:unauthorized": None,
            "urn:ietf:params:acme:error:userActionRequired": None,
            "urn:ietf:params:acme:error:alreadyRevoked": None,
            "notImplementedYet": "we are not that far. Stay tuned",
        }
        if message and message in error_dic:
            result = error_dic[message]
        else:
            result = None
        return result

    def enrich_error(self, message, detail=None):
        """put some more content into the error messgae"""
        self.logger.debug("Error.enrich_error()")
        error_message = self._acme_errormessage(message)

        if message and error_message:
            detail = "{0}: {1}".format(error_message, detail)
        elif error_message:
            detail = "{0}{1}".format(error_message, detail)

        return detail



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""unittests for openssl_ca_handler"""

# pylint: disable=C0415, R0904, W0212
import sys
import os
import unittest
from unittest.mock import patch, Mock, MagicMock
import requests
import base64

sys.path.insert(0, ".")
sys.path.insert(1, "..")


class TestACMEHandler(unittest.TestCase):
    """test class for cgi_handler"""

    def setUp(self):
        """setup unittest"""
        import logging
        from acme2certifier.cahandlers.asa_ca_handler import CAhandler

        logging.basicConfig(level=logging.CRITICAL)
        self.logger = logging.getLogger("test_a2c")
        self.cahandler = CAhandler(False, self.logger)
        self.dir_path = os.path.dirname(os.path.realpath(__file__))
        self.maxDiff = None

    def test_001_default(self):
        """default test which always passes"""
        self.assertEqual("foo", "foo")

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._config_load")
    def test_002__enter__(self, mock_cfg):
        """test enter  called"""
        mock_cfg.return_value = True
        self.cahandler.__enter__()
        self.assertTrue(mock_cfg.called)

    def test_003_poll(self):
        """test polling"""
        self.assertEqual(
            ("Method not implemented.", None, None, "poll_identifier", False),
            self.cahandler.poll("cert_name", "poll_identifier", "csr"),
        )

    def test_004_trigger(self):
        """test polling"""
        self.assertEqual(
            ("Method not implemented.", None, None), self.cahandler.trigger("payload")
        )

    @patch("acme2certifier.cahandlers.asa_ca_handler.load_config")
    def test_005_config_load(self, mock_config_load):
        """test _config_load"""
        mock_config_load.return_value = {"CAhandler": {"api_host": "api_host"}}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_user has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_password has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_key has not been not set",
            lcm.output,
        )
        self.assertEqual("api_host", self.cahandler.api_host)
        self.assertFalse(self.cahandler.api_user)
        self.assertFalse(self.cahandler.api_password)
        self.assertFalse(self.cahandler.api_key)
        self.assertFalse(self.cahandler.ca_bundle)
        self.assertFalse(self.cahandler.proxy)
        self.assertEqual(10, self.cahandler.request_timeout)
        self.assertEqual(30, self.cahandler.cert_validity_days)
        self.assertFalse(self.cahandler.header_info_field)

    @patch("acme2certifier.cahandlers.asa_ca_handler.load_config")
    def test_006_config_load(self, mock_config_load):
        """test _config_load"""
        mock_config_load.return_value = {"CAhandler": {"api_user": "api_user"}}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_host has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_key has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_password has not been not set",
            lcm.output,
        )
        self.assertFalse(self.cahandler.api_host)
        self.assertEqual("api_user", self.cahandler.api_user)
        self.assertFalse(self.cahandler.api_password)
        self.assertFalse(self.cahandler.api_key)
        self.assertFalse(self.cahandler.ca_bundle)
        self.assertFalse(self.cahandler.proxy)
        self.assertEqual(10, self.cahandler.request_timeout)
        self.assertEqual(30, self.cahandler.cert_validity_days)
        self.assertFalse(self.cahandler.header_info_field)

    @patch("acme2certifier.cahandlers.asa_ca_handler.load_config")
    def test_007_config_load(self, mock_config_load):
        """test _config_load"""
        mock_config_load.return_value = {"CAhandler": {"api_password": "api_password"}}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_host has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_user has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_key has not been not set",
            lcm.output,
        )
        self.assertFalse(self.cahandler.api_host)
        self.assertFalse(self.cahandler.api_user)
        self.assertEqual("api_password", self.cahandler.api_password)
        self.assertFalse(self.cahandler.api_key)
        self.assertFalse(self.cahandler.ca_bundle)
        self.assertFalse(self.cahandler.proxy)
        self.assertEqual(10, self.cahandler.request_timeout)
        self.assertEqual(30, self.cahandler.cert_validity_days)
        self.assertFalse(self.cahandler.header_info_field)

    @patch("acme2certifier.cahandlers.asa_ca_handler.load_config")
    def test_008_config_load(self, mock_config_load):
        """test _config_load"""
        mock_config_load.return_value = {"CAhandler": {"api_key": "api_key"}}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_host has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_user has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_password has not been not set",
            lcm.output,
        )
        self.assertFalse(self.cahandler.api_host)
        self.assertFalse(self.cahandler.api_user)
        self.assertFalse(self.cahandler.api_password)
        self.assertEqual("api_key", self.cahandler.api_key)
        self.assertFalse(self.cahandler.ca_bundle)
        self.assertFalse(self.cahandler.proxy)
        self.assertEqual(10, self.cahandler.request_timeout)
        self.assertEqual(30, self.cahandler.cert_validity_days)
        self.assertFalse(self.cahandler.header_info_field)

    @patch("acme2certifier.cahandlers.asa_ca_handler.load_config")
    def test_009_config_load(self, mock_config_load):
        """test _config_load"""
        mock_config_load.return_value = {
            "CAhandler": {
                "api_host": "api_host",
                "api_user": "api_user",
                "api_password": "api_password",
                "api_key": "api_key",
            }
        }
        self.cahandler._config_load()
        self.assertEqual("api_host", self.cahandler.api_host)
        self.assertEqual("api_user", self.cahandler.api_user)
        self.assertEqual("api_password", self.cahandler.api_password)
        self.assertEqual("api_key", self.cahandler.api_key)
        self.assertFalse(self.cahandler.proxy)
        self.assertEqual(10, self.cahandler.request_timeout)
        self.assertEqual(30, self.cahandler.cert_validity_days)
        self.assertFalse(self.cahandler.header_info_field)

    @patch("acme2certifier.cahandlers.asa_ca_handler.load_config")
    def test_010_config_load(self, mock_config_load):
        """test _config_load"""
        mock_config_load.return_value = {"foo": "bar"}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_host has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_user has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_key has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_password has not been not set",
            lcm.output,
        )
        self.assertFalse(self.cahandler.api_host)
        self.assertFalse(self.cahandler.api_user)
        self.assertFalse(self.cahandler.api_password)
        self.assertFalse(self.cahandler.api_key)
        self.assertFalse(self.cahandler.ca_bundle)
        self.assertFalse(self.cahandler.proxy)
        self.assertEqual(10, self.cahandler.request_timeout)
        self.assertEqual(30, self.cahandler.cert_validity_days)
        self.assertFalse(self.cahandler.header_info_field)

    @patch("acme2certifier.cahandlers.asa_ca_handler.load_config")
    def test_011_config_load(self, mock_config_load):
        """test _config_load"""
        mock_config_load.return_value = {"CAhandler": {"request_timeout": 20}}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_host has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_user has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_key has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_password has not been not set",
            lcm.output,
        )
        self.assertFalse(self.cahandler.api_host)
        self.assertFalse(self.cahandler.api_user)
        self.assertFalse(self.cahandler.api_password)
        self.assertFalse(self.cahandler.api_key)
        self.assertFalse(self.cahandler.ca_bundle)
        self.assertFalse(self.cahandler.proxy)
        self.assertEqual(20, self.cahandler.request_timeout)
        self.assertEqual(30, self.cahandler.cert_validity_days)
        self.assertFalse(self.cahandler.header_info_field)

    @patch("acme2certifier.cahandlers.asa_ca_handler.load_config")
    def test_012_config_load(self, mock_config_load):
        """test _config_load"""
        mock_config_load.return_value = {"CAhandler": {"request_timeout": "aa"}}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_host has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_user has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_key has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_password has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:request_timeout parameter is not an integer. Error: invalid literal for int() with base 10: 'aa'",
            lcm.output,
        )
        self.assertFalse(self.cahandler.api_host)
        self.assertFalse(self.cahandler.api_user)
        self.assertFalse(self.cahandler.api_password)
        self.assertFalse(self.cahandler.api_key)
        self.assertFalse(self.cahandler.ca_bundle)
        self.assertFalse(self.cahandler.proxy)
        self.assertEqual(10, self.cahandler.request_timeout)
        self.assertEqual(30, self.cahandler.cert_validity_days)
        self.assertFalse(self.cahandler.header_info_field)

    @patch("acme2certifier.cahandlers.asa_ca_handler.load_config")
    def test_013_config_load(self, mock_config_load):
        """test _config_load"""
        mock_config_load.return_value = {"CAhandler": {"cert_validity_days": 10}}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_host has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_user has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_key has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_password has not been not set",
            lcm.output,
        )
        # self.assertIn('ERROR:test_a2c:CAhandler._config_load(): request_timeout not an integer', lcm.output)
        self.assertFalse(self.cahandler.api_host)
        self.assertFalse(self.cahandler.api_user)
        self.assertFalse(self.cahandler.api_password)
        self.assertFalse(self.cahandler.api_key)
        self.assertFalse(self.cahandler.ca_bundle)
        self.assertFalse(self.cahandler.proxy)
        self.assertEqual(10, self.cahandler.request_timeout)
        self.assertEqual(10, self.cahandler.cert_validity_days)
        self.assertFalse(self.cahandler.header_info_field)

    @patch("acme2certifier.cahandlers.asa_ca_handler.load_config")
    def test_014_config_load(self, mock_config_load):
        """test _config_load"""
        mock_config_load.return_value = {"CAhandler": {"cert_validity_days": "aa"}}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_host has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_user has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_key has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_password has not been not set",
            lcm.output,
        )
        self.assertFalse(self.cahandler.api_host)
        self.assertFalse(self.cahandler.api_user)
        self.assertFalse(self.cahandler.api_password)
        self.assertFalse(self.cahandler.api_key)
        self.assertFalse(self.cahandler.ca_bundle)
        self.assertFalse(self.cahandler.proxy)
        self.assertEqual(10, self.cahandler.request_timeout)
        self.assertEqual(30, self.cahandler.cert_validity_days)
        self.assertFalse(self.cahandler.header_info_field)

    @patch("acme2certifier.cahandlers.asa_ca_handler.load_config")
    def test_015_config_load(self, mock_config_load):
        """test _config_load"""
        mock_config_load.return_value = {"CAhandler": {"ca_bundle": "aa"}}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_host has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_user has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_key has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_password has not been not set",
            lcm.output,
        )
        self.assertFalse(self.cahandler.api_host)
        self.assertFalse(self.cahandler.api_user)
        self.assertFalse(self.cahandler.api_password)
        self.assertFalse(self.cahandler.api_key)
        self.assertEqual("aa", self.cahandler.ca_bundle)
        self.assertFalse(self.cahandler.proxy)
        self.assertEqual(10, self.cahandler.request_timeout)
        self.assertEqual(30, self.cahandler.cert_validity_days)
        self.assertFalse(self.cahandler.header_info_field)

    @patch("acme2certifier.cahandlers.asa_ca_handler.load_config")
    def test_016_config_load(self, mock_config_load):
        """test _config_load"""
        mock_config_load.return_value = {"CAhandler": {"ca_bundle": "False"}}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_host has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_user has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_key has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_password has not been not set",
            lcm.output,
        )
        self.assertFalse(self.cahandler.api_host)
        self.assertFalse(self.cahandler.api_user)
        self.assertFalse(self.cahandler.api_password)
        self.assertFalse(self.cahandler.api_key)
        self.assertFalse(self.cahandler.ca_bundle)
        self.assertFalse(self.cahandler.proxy)
        self.assertEqual(10, self.cahandler.request_timeout)
        self.assertEqual(30, self.cahandler.cert_validity_days)
        self.assertFalse(self.cahandler.header_info_field)

    @patch("acme2certifier.cahandlers.asa_ca_handler.load_config")
    def test_017_config_load(self, mock_config_load):
        """test _config_load"""
        mock_config_load.return_value = {"Order": {"header_info_list": '["foo"]'}}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_load()
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_host has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_user has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_key has not been not set",
            lcm.output,
        )
        self.assertIn(
            "ERROR:test_a2c:Configuration incomplete. Variable api_password has not been not set",
            lcm.output,
        )
        self.assertFalse(self.cahandler.api_host)
        self.assertFalse(self.cahandler.api_user)
        self.assertFalse(self.cahandler.api_password)
        self.assertFalse(self.cahandler.api_key)
        self.assertFalse(self.cahandler.ca_bundle)
        self.assertFalse(self.cahandler.proxy)
        self.assertEqual(10, self.cahandler.request_timeout)
        self.assertEqual(30, self.cahandler.cert_validity_days)
        self.assertEqual("foo", self.cahandler.header_info_field)

    @patch.object(requests, "post")
    def test_018__api_post(self, mock_req):
        """test _api_post()"""
        mockresponse = Mock()
        mockresponse.status_code = "status_code"
        mockresponse.json = lambda: {"foo": "bar"}
        mock_req.return_value = mockresponse
        self.assertEqual(
            ("status_code", {"foo": "bar"}), self.cahandler._api_post("url", "data")
        )

    @patch("requests.post")
    def test_019__api_post(self, mock_req):
        """test _api_post()"""
        mockresponse = Mock()
        mockresponse.status_code = "status_code"
        mockresponse.json = "aaaa"
        mock_req.return_value = mockresponse
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.assertEqual(
                ("status_code", "'str' object is not callable"),
                self.cahandler._api_post("url", "data"),
            )
        self.assertIn(
            "ERROR:test_a2c:Could not parse the response for an API post() request: 'str' object is not callable",
            lcm.output,
        )

    @patch("requests.post")
    def test_020__api_post(self, mock_req):
        """test _api_post()"""
        mockresponse = Mock()
        mockresponse.status_code = "status_code"
        mockresponse.text = None
        mock_req.return_value = mockresponse
        self.assertEqual(("status_code", None), self.cahandler._api_post("url", "data"))

    @patch("requests.post")
    def test_021__api_post(self, mock_req):
        """test _api_post(="""
        self.cahandler.api_host = "api_host"
        self.cahandler.auth = "auth"
        mock_req.side_effect = Exception("exc_api_post")
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.assertEqual(
                (500, "exc_api_post"), self.cahandler._api_post("url", "data")
            )
        self.assertIn(
            "ERROR:test_a2c:API post() request returned an error: exc_api_post",
            lcm.output,
        )

    @patch.object(requests, "get")
    def test_022__api_get(self, mock_req):
        """test _api_get()"""
        mockresponse = Mock()
        mockresponse.status_code = "status_code"
        mockresponse.json = lambda: {"foo": "bar"}
        mock_req.return_value = mockresponse
        self.assertEqual(
            ("status_code", {"foo": "bar"}), self.cahandler._api_get("url")
        )

    @patch("requests.get")
    def test_023__api_get(self, mock_req):
        """test _api_get()"""
        mockresponse = Mock()
        mockresponse.status_code = "status_code"
        mockresponse.json = "aaaa"
        mock_req.return_value = mockresponse
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.assertEqual(
                ("status_code", "'str' object is not callable"),
                self.cahandler._api_get("url"),
            )
        self.assertIn(
            "ERROR:test_a2c:Could not parse the response for an API get() request: 'str' object is not callable",
            lcm.output,
        )

    @patch("requests.get")
    def test_024__api_get(self, mock_req):
        """test _api_get()"""
        self.cahandler.api_host = "api_host"
        self.cahandler.auth = "auth"
        mock_req.side_effect = Exception("exc_api_get")
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.assertEqual((500, "exc_api_get"), self.cahandler._api_get("url"))
        self.assertIn(
            "ERROR:test_a2c:API get() request returned error: exc_api_get",
            lcm.output,
        )

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._api_get")
    def test_025__issuers_list(self, mock_get):
        """test _issuers_list()"""
        mock_get.return_value = (200, "content")
        self.assertEqual("content", self.cahandler._issuers_list())

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._api_get")
    def test_026__profiles_list(self, mock_get):
        """test _profiles_list()"""
        self.cahandler.ca_name = "ca_name"
        mock_get.return_value = (200, "content")
        self.assertEqual("content", self.cahandler._profiles_list())

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._api_get")
    def test_027__certificates_list(self, mock_get):
        """test _profiles_list()"""
        self.cahandler.ca_name = "ca_name"
        mock_get.return_value = (200, "content")
        self.assertEqual("content", self.cahandler._certificates_list())

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._api_post")
    def test_028_cert_status_get(self, mock_req):
        """test _profiles_list()"""
        self.cahandler.ca_name = "ca_name"
        mock_req.return_value = ("status_code", {"foo": "bar"})
        self.assertEqual(
            {"foo": "bar", "code": "status_code"},
            self.cahandler._cert_status_get("cert"),
        )

    @patch("acme2certifier.cahandlers.asa_ca_handler.csr_san_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.csr_cn_get")
    def test_029__csr_cn_get(self, mock_cn, mock_san):
        """test _csr_cn_get()"""
        mock_cn.return_value = "cn"
        mock_san.return_value = ["san0", "san1"]
        self.assertEqual("cn", self.cahandler._csr_cn_get("csr"))
        self.assertFalse(mock_san.called)

    @patch("acme2certifier.cahandlers.asa_ca_handler.csr_san_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.csr_cn_get")
    def test_030__csr_cn_get(self, mock_cn, mock_san):
        """test _csr_cn_get()"""
        mock_cn.return_value = None
        mock_san.return_value = ["dns:san0", "dns:san1"]
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.assertEqual("san0", self.cahandler._csr_cn_get("csr"))
        self.assertIn("INFO:test_a2c:CN not found in CSR", lcm.output)
        self.assertIn(
            "INFO:test_a2c:CN not found in CSR. Using first SAN entry as CN: san0",
            lcm.output,
        )
        self.assertTrue(mock_san.called)

    @patch("acme2certifier.cahandlers.asa_ca_handler.csr_san_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.csr_cn_get")
    def test_031__csr_cn_get(self, mock_cn, mock_san):
        """test _csr_cn_get()"""
        mock_cn.return_value = None
        mock_san.return_value = None
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.assertEqual(None, self.cahandler._csr_cn_get("csr"))
        self.assertIn("INFO:test_a2c:CN not found in CSR", lcm.output)
        self.assertIn(
            "ERROR:test_a2c:CN not found in CSR. No SAN entries found",
            lcm.output,
        )
        self.assertTrue(mock_san.called)

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._issuers_list")
    def test_032_issuer_verify(self, mock_list):
        """_issuer_verify()"""
        self.cahandler.ca_name = "ca_name"
        mock_list.return_value = {"issuers": ["1", "2", "ca_name"]}
        self.assertFalse(self.cahandler._issuer_verify())

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._issuers_list")
    def test_033_issuer_verify(self, mock_list):
        """_issuer_verify()"""
        self.cahandler.ca_name = "ca_name"
        mock_list.return_value = {"issuers": ["1", "2", "3"]}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.assertEqual("CA ca_name not found", self.cahandler._issuer_verify())
        self.assertIn(
            "ERROR:test_a2c:CAhandler.enroll(): CA ca_name not found", lcm.output
        )

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._issuers_list")
    def test_034_issuer_verify(self, mock_list):
        """_issuer_verify()"""
        self.cahandler.ca_name = "ca_name"
        mock_list.return_value = {"foo": "bar"}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.assertEqual("Malformed response", self.cahandler._issuer_verify())
        self.assertIn(
            'ERROR:test_a2c:Malformed response. "issuers" key not found',
            lcm.output,
        )

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._profiles_list")
    def test_035_profile_verify(self, mock_list):
        """_profile_verify()"""
        self.cahandler.profile_name = "profile_name"
        mock_list.return_value = {"profiles": ["1", "2", "profile_name"]}
        self.assertFalse(self.cahandler._profile_verify())

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._profiles_list")
    def test_036_profile_verify(self, mock_list):
        """_profile_verify()"""
        self.cahandler.profile_name = "profile_name"
        mock_list.return_value = {"profiles": ["1", "2", "3"]}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.assertEqual(
                "Profile profile_name not found", self.cahandler._profile_verify()
            )
        self.assertIn(
            "ERROR:test_a2c:Profile profile_name not found",
            lcm.output,
        )

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._profiles_list")
    def test_037_profile_verify(self, mock_list):
        """_profile_verify()"""
        self.cahandler.ca_name = "ca_name"
        mock_list.return_value = {"foo": "bar"}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.assertEqual("Malformed response", self.cahandler._profile_verify())
        self.assertIn(
            'ERROR:test_a2c:Malformed response. "profiles" key not found',
            lcm.output,
        )

    @patch("acme2certifier.cahandlers.asa_ca_handler.uts_to_date_utc")
    @patch("acme2certifier.cahandlers.asa_ca_handler.uts_now")
    def test_038__validity_dates_get(self, mock_now, mock_utc):
        """test _validity_dates_get()"""
        mock_now.return_value = 10
        mock_utc.side_effect = ["date1", "date2"]
        self.assertEqual(("date1", "date2"), self.cahandler._validity_dates_get())
        self.assertTrue(mock_now.called)

    @patch("acme2certifier.cahandlers.asa_ca_handler.convert_byte_to_string")
    @patch("acme2certifier.cahandlers.asa_ca_handler.cert_der2pem")
    @patch("acme2certifier.cahandlers.asa_ca_handler.b64_decode")
    def test_039__pem_cert_chain_generate(self, mock_dec, mock_d2p, mock_b2s):
        """test _pem_cert_chain_generate()"""
        mock_b2s.return_value = "cert"
        self.assertEqual(
            "certcert", self.cahandler._pem_cert_chain_generate(["cert", "chain"])
        )

    def test_040__pem_cert_chain_generate(self):
        """test _pem_cert_chain_generate()"""
        cert_list = [
            "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"
        ]
        result = """-----BEGIN CERTIFICATE-----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-----END CERTIFICATE-----
"""
        self.assertEqual(result, self.cahandler._pem_cert_chain_generate(cert_list))

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._pem_cert_chain_generate")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._api_get")
    def test_041___issuer_chain_get(self, mock_req, mock_pem):
        """test _issuer_chain_get()"""
        mock_req.return_value = ("code", {"certs": ["bar", "foo"]})
        mock_pem.return_value = "issuer_chain"
        self.cahandler.ca_name = "ca_name"
        self.assertEqual("issuer_chain", self.cahandler._issuer_chain_get())
        self.assertTrue(mock_pem.called)

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._pem_cert_chain_generate")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._api_get")
    def test_042___issuer_chain_get(self, mock_req, mock_pem):
        """test _issuer_chain_get()"""
        mock_req.return_value = ("code", {"foobar": ["bar", "foo"]})
        mock_pem.return_value = "issuer_chain"
        self.cahandler.ca_name = "ca_name"
        self.assertFalse(self.cahandler._issuer_chain_get())
        self.assertFalse(mock_pem.called)

    @patch("acme2certifier.cahandlers.asa_ca_handler.enrollment_config_log")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.asa_ca_handler.convert_byte_to_string")
    @patch("acme2certifier.cahandlers.asa_ca_handler.cert_der2pem")
    @patch("acme2certifier.cahandlers.asa_ca_handler.b64_decode")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._validity_dates_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._csr_cn_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.csr_pubkey_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._issuer_chain_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._issuer_verify")
    @patch("acme2certifier.cahandlers.asa_ca_handler.eab_profile_header_info_check")
    def test_043_enroll(
        self,
        mock_pv,
        mock_iv,
        mock_icg,
        mock_cpg,
        mockccg,
        mock_vdg,
        mock_b64,
        mock_d2p,
        mock_b2s,
        mock_post,
        mock_ecl,
    ):
        """test enroll()"""
        mock_iv.return_value = None
        mock_pv.return_value = "pv_error"
        mock_icg.return_value = "issuer_chain"
        mock_vdg.return_value = ("date1", "date2")
        mock_post.return_value = (200, "cert")
        mock_b2s.return_value = "bcert"
        self.cahandler.header_info_field = "foo"
        self.assertEqual(("pv_error", None, None, None), self.cahandler.enroll("csr"))
        self.assertTrue(mock_pv.called)
        self.assertFalse(mock_iv.called)
        self.assertFalse(mock_icg.called)
        self.assertFalse(mock_cpg.called)
        self.assertFalse(mockccg.called)
        self.assertFalse(mock_vdg.called)
        self.assertFalse(mock_b64.called)
        self.assertFalse(mock_post.called)
        self.assertFalse(mock_b2s.called)
        self.assertFalse(mock_d2p.called)
        self.assertFalse(mock_ecl.called)

    @patch("acme2certifier.cahandlers.asa_ca_handler.enrollment_config_log")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.asa_ca_handler.convert_byte_to_string")
    @patch("acme2certifier.cahandlers.asa_ca_handler.cert_der2pem")
    @patch("acme2certifier.cahandlers.asa_ca_handler.b64_decode")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._validity_dates_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._csr_cn_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.csr_pubkey_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._issuer_chain_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._profile_verify")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._issuer_verify")
    def test_044_enroll(
        self,
        mock_iv,
        mock_pv,
        mock_icg,
        mock_cpg,
        mockccg,
        mock_vdg,
        mock_b64,
        mock_d2p,
        mock_b2s,
        mock_post,
        mock_ecl,
    ):
        """test enroll()"""
        mock_iv.return_value = None
        mock_pv.return_value = None
        mock_icg.return_value = "issuer_chain"
        mock_vdg.return_value = ("date1", "date2")
        mock_post.return_value = (200, "cert")
        mock_b2s.return_value = "bcert"
        self.cahandler.header_info_field = "foo"
        self.cahandler.enrollment_config_log = True
        self.assertEqual(
            (None, "bcertissuer_chain", "cert", None), self.cahandler.enroll("csr")
        )
        self.assertTrue(mock_iv.called)
        self.assertTrue(mock_pv.called)
        self.assertTrue(mock_icg.called)
        self.assertTrue(mock_cpg.called)
        self.assertTrue(mockccg.called)
        self.assertTrue(mock_vdg.called)
        self.assertTrue(mock_b64.called)
        self.assertTrue(mock_post.called)
        self.assertTrue(mock_b2s.called)
        self.assertTrue(mock_d2p.called)
        self.assertTrue(mock_ecl.called)

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.asa_ca_handler.convert_byte_to_string")
    @patch("acme2certifier.cahandlers.asa_ca_handler.cert_der2pem")
    @patch("acme2certifier.cahandlers.asa_ca_handler.b64_decode")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._validity_dates_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._csr_cn_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.csr_pubkey_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._issuer_chain_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._profile_verify")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._issuer_verify")
    def test_045_enroll(
        self,
        mock_iv,
        mock_pv,
        mock_icg,
        mock_cpg,
        mockccg,
        mock_vdg,
        mock_b64,
        mock_d2p,
        mock_b2s,
        mock_post,
    ):
        """test enroll()"""
        mock_iv.return_value = None
        mock_pv.return_value = None
        mock_icg.return_value = "issuer_chain"
        mock_vdg.return_value = ("date1", "date2")
        mock_post.return_value = (200, "cert")
        mock_b2s.return_value = "bcert"
        self.assertEqual(
            (None, "bcertissuer_chain", "cert", None), self.cahandler.enroll("csr")
        )
        self.assertTrue(mock_iv.called)
        self.assertTrue(mock_pv.called)
        self.assertTrue(mock_icg.called)
        self.assertTrue(mock_cpg.called)
        self.assertTrue(mockccg.called)
        self.assertTrue(mock_vdg.called)
        self.assertTrue(mock_b64.called)
        self.assertTrue(mock_post.called)
        self.assertTrue(mock_b2s.called)
        self.assertTrue(mock_d2p.called)

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.asa_ca_handler.convert_byte_to_string")
    @patch("acme2certifier.cahandlers.asa_ca_handler.cert_der2pem")
    @patch("acme2certifier.cahandlers.asa_ca_handler.b64_decode")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._validity_dates_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._csr_cn_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.csr_pubkey_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._issuer_chain_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._profile_verify")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._issuer_verify")
    def test_046_enroll(
        self,
        mock_iv,
        mock_pv,
        mock_icg,
        mock_cpg,
        mockccg,
        mock_vdg,
        mock_b64,
        mock_d2p,
        mock_b2s,
        mock_post,
    ):
        """test enroll()"""
        mock_iv.return_value = "mock_iv"
        mock_pv.return_value = None
        mock_icg.return_value = "issuer_chain"
        mock_vdg.return_value = ("date1", "date2")
        mock_post.return_value = ("code", "cert")
        mock_b2s.return_value = "bcert"
        self.assertEqual(("mock_iv", None, None, None), self.cahandler.enroll("csr"))
        self.assertTrue(mock_iv.called)
        self.assertFalse(mock_pv.called)
        self.assertFalse(mock_icg.called)
        self.assertFalse(mock_cpg.called)
        self.assertFalse(mockccg.called)
        self.assertFalse(mock_vdg.called)
        self.assertFalse(mock_b64.called)
        self.assertFalse(mock_post.called)
        self.assertFalse(mock_b2s.called)
        self.assertFalse(mock_d2p.called)

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.asa_ca_handler.convert_byte_to_string")
    @patch("acme2certifier.cahandlers.asa_ca_handler.cert_der2pem")
    @patch("acme2certifier.cahandlers.asa_ca_handler.b64_decode")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._validity_dates_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._csr_cn_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.csr_pubkey_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._issuer_chain_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._profile_verify")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._issuer_verify")
    def test_047_enroll(
        self,
        mock_iv,
        mock_pv,
        mock_icg,
        mock_cpg,
        mockccg,
        mock_vdg,
        mock_b64,
        mock_d2p,
        mock_b2s,
        mock_post,
    ):
        """test enroll()"""
        mock_iv.return_value = None
        mock_pv.return_value = "mock_pv"
        mock_icg.return_value = "issuer_chain"
        mock_vdg.return_value = ("date1", "date2")
        mock_post.return_value = ("code", "cert")
        mock_b2s.return_value = "bcert"
        self.assertEqual(("mock_pv", None, None, None), self.cahandler.enroll("csr"))
        self.assertTrue(mock_iv.called)
        self.assertTrue(mock_pv.called)
        self.assertFalse(mock_icg.called)
        self.assertFalse(mock_cpg.called)
        self.assertFalse(mockccg.called)
        self.assertFalse(mock_vdg.called)
        self.assertFalse(mock_b64.called)
        self.assertFalse(mock_post.called)
        self.assertFalse(mock_b2s.called)
        self.assertFalse(mock_d2p.called)

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.asa_ca_handler.convert_byte_to_string")
    @patch("acme2certifier.cahandlers.asa_ca_handler.cert_der2pem")
    @patch("acme2certifier.cahandlers.asa_ca_handler.b64_decode")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._validity_dates_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._csr_cn_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.csr_pubkey_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._issuer_chain_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._profile_verify")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._issuer_verify")
    def test_048_enroll(
        self,
        mock_iv,
        mock_pv,
        mock_icg,
        mock_cpg,
        mockccg,
        mock_vdg,
        mock_b64,
        mock_d2p,
        mock_b2s,
        mock_post,
    ):
        """test enroll()"""
        mock_iv.return_value = None
        mock_pv.return_value = None
        mock_icg.return_value = "issuer_chain"
        mock_vdg.return_value = ("date1", "date2")
        mock_post.return_value = (500, "cert")
        mock_b2s.return_value = "bcert"
        self.assertEqual(
            ("Enrollment failed", None, None, None), self.cahandler.enroll("csr")
        )
        self.assertTrue(mock_iv.called)
        self.assertTrue(mock_pv.called)
        self.assertTrue(mock_icg.called)
        self.assertTrue(mock_cpg.called)
        self.assertTrue(mockccg.called)
        self.assertTrue(mock_vdg.called)
        self.assertFalse(mock_b64.called)
        self.assertTrue(mock_post.called)
        self.assertFalse(mock_b2s.called)
        self.assertFalse(mock_d2p.called)

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.asa_ca_handler.convert_byte_to_string")
    @patch("acme2certifier.cahandlers.asa_ca_handler.cert_der2pem")
    @patch("acme2certifier.cahandlers.asa_ca_handler.b64_decode")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._validity_dates_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._csr_cn_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.csr_pubkey_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._issuer_chain_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._profile_verify")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._issuer_verify")
    def test_049_enroll(
        self,
        mock_iv,
        mock_pv,
        mock_icg,
        mock_cpg,
        mockccg,
        mock_vdg,
        mock_b64,
        mock_d2p,
        mock_b2s,
        mock_post,
    ):
        """test enroll()"""
        mock_iv.return_value = None
        mock_pv.return_value = None
        mock_icg.return_value = "issuer_chain"
        mock_vdg.return_value = ("date1", "date2")
        mock_post.return_value = (500, "cert")
        mock_b2s.return_value = "bcert"
        mock_cpg.return_value = None
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.assertEqual(
                ("Enrollment failed", None, None, None), self.cahandler.enroll("csr")
            )
        self.assertIn(
            "ERROR:test_a2c:Could not extract the public key from CSR",
            lcm.output,
        )
        self.assertTrue(mock_iv.called)
        self.assertTrue(mock_pv.called)
        self.assertTrue(mock_icg.called)
        self.assertTrue(mock_cpg.called)
        self.assertFalse(mockccg.called)
        self.assertFalse(mock_vdg.called)
        self.assertFalse(mock_b64.called)
        self.assertFalse(mock_post.called)
        self.assertFalse(mock_b2s.called)
        self.assertFalse(mock_d2p.called)

    @patch("acme2certifier.cahandlers.asa_ca_handler.eab_profile_revocation_check")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.asa_ca_handler.cert_ski_get")
    def test_050_revoke(self, mock_ski, mock_post, mock_epr):
        """test revoke()"""
        self.cahandler.ca_name = "ca_name"
        mock_ski.return_value = "serial"
        mock_post.return_value = ("code", None)
        self.assertEqual(("code", None, None), self.cahandler.revoke("cert"))
        self.assertTrue(mock_ski.called)
        self.assertTrue(mock_post.called)
        self.assertFalse(mock_epr.called)

    @patch("acme2certifier.cahandlers.asa_ca_handler.eab_profile_revocation_check")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.asa_ca_handler.cert_ski_get")
    def test_051_revoke(self, mock_ski, mock_post, mock_epr):
        """test revoke()"""
        self.cahandler.ca_name = "ca_name"
        mock_ski.return_value = "serial"
        mock_post.return_value = ("code", None)
        self.cahandler.eab_profiling = True
        self.assertEqual(("code", None, None), self.cahandler.revoke("cert"))
        self.assertTrue(mock_ski.called)
        self.assertTrue(mock_post.called)
        self.assertTrue(mock_epr.called)

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.asa_ca_handler.cert_ski_get")
    def test_052_revoke(self, mock_ski, mock_post):
        """test revoke()"""
        self.cahandler.ca_name = "ca_name"
        mock_ski.return_value = "mock_ski"
        mock_post.return_value = ("code", {"message": "message"})
        self.assertEqual(
            ("code", "urn:ietf:params:acme:error:serverInternal", "message"),
            self.cahandler.revoke("cert"),
        )
        self.assertTrue(mock_ski.called)
        self.assertTrue(mock_post.called)

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.asa_ca_handler.cert_ski_get")
    def test_053_revoke(self, mock_ski, mock_post):
        """test revoke()"""
        self.cahandler.ca_name = "ca_name"
        mock_ski.return_value = "ski"
        mock_post.return_value = ("code", {"Message": "Message"})
        self.assertEqual(
            ("code", "urn:ietf:params:acme:error:serverInternal", "Message"),
            self.cahandler.revoke("cert"),
        )
        self.assertTrue(mock_ski.called)
        self.assertTrue(mock_post.called)

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._api_post")
    @patch("acme2certifier.cahandlers.asa_ca_handler.cert_ski_get")
    def test_054_revoke(self, mock_ski, mock_post):
        """test revoke()"""
        self.cahandler.ca_name = "ca_name"
        mock_ski.return_value = "ski"
        mock_post.return_value = ("code", {"foo": "bar"})
        self.assertEqual(
            ("code", "urn:ietf:params:acme:error:serverInternal", "Unknown error"),
            self.cahandler.revoke("cert"),
        )
        self.assertTrue(mock_ski.called)
        self.assertTrue(mock_post.called)

    @patch.dict("os.environ", {"api_user_var": "user_var"})
    def test_055_config_user_load(self):
        """test _config_load - load template with user variable"""
        config_dic = {"api_user_variable": "api_user_var"}
        self.cahandler._config_user_load(config_dic)
        self.assertEqual("user_var", self.cahandler.api_user)
        self.assertFalse(self.cahandler.profile_name)

    @patch.dict("os.environ", {"api_user_var": "user_var"})
    def test_056_config_user_load(self):
        """test _config_load - load template with user variable"""
        config_dic = {"api_user_variable": "does_not_exist"}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_user_load(config_dic)
        self.assertFalse(self.cahandler.api_user)
        self.assertIn(
            "ERROR:test_a2c:Could not load user_variable: does_not_exist",
            lcm.output,
        )
        self.assertFalse(self.cahandler.profile_name)

    @patch.dict("os.environ", {"api_user_var": "user_var"})
    def test_057_config_user_load(self):
        """test _config_load - load template with user variable"""
        config_dic = {"api_user_variable": "api_user_var", "api_user": "api_user"}
        self.cahandler._config_user_load(config_dic)
        # with self.assertLogs('test_a2c', level='INFO') as lcm:
        self.assertEqual("api_user", self.cahandler.api_user)
        # self.assertIn("foo", lcm.output)
        self.assertFalse(self.cahandler.profile_name)

    @patch.dict("os.environ", {"api_host_var": "host_var"})
    def test_058_config_host_load(self):
        """test _config_load - load template with host variable"""
        config_dic = {"api_host_variable": "api_host_var"}
        self.cahandler._config_host_load(config_dic)
        self.assertEqual("host_var", self.cahandler.api_host)
        self.assertFalse(self.cahandler.profile_name)

    @patch.dict("os.environ", {"api_host_var": "host_var"})
    def test_059_config_host_load(self):
        """test _config_load - load template with host variable"""
        config_dic = {"api_host_variable": "does_not_exist"}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_host_load(config_dic)
        self.assertFalse(self.cahandler.api_host)
        self.assertIn(
            "ERROR:test_a2c:Could not load host_variable: does_not_exist",
            lcm.output,
        )
        self.assertFalse(self.cahandler.profile_name)

    @patch.dict("os.environ", {"api_host_var": "host_var"})
    def test_060_config_host_load(self):
        """test _config_load - load template with host variable"""
        config_dic = {"api_host_variable": "api_host_var", "api_host": "api_host"}
        self.cahandler._config_host_load(config_dic)
        # with self.assertLogs('test_a2c', level='INFO') as lcm:
        self.assertEqual("api_host", self.cahandler.api_host)
        # self.assertIn("foo", lcm.output)
        self.assertFalse(self.cahandler.profile_name)

    @patch.dict("os.environ", {"api_key_var": "key_var"})
    def test_061_config_key_load(self):
        """test _config_load - load template with key variable"""
        config_dic = {"api_key_variable": "api_key_var"}
        self.cahandler._config_key_load(config_dic)
        self.assertEqual("key_var", self.cahandler.api_key)
        self.assertFalse(self.cahandler.profile_name)

    @patch.dict("os.environ", {"api_key_var": "key_var"})
    def test_062_config_key_load(self):
        """test _config_load - load template with key variable"""
        config_dic = {"api_key_variable": "does_not_exist"}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_key_load(config_dic)
        self.assertFalse(self.cahandler.api_key)
        self.assertIn(
            "ERROR:test_a2c:Could not load key_variable: does_not_exist",
            lcm.output,
        )
        self.assertFalse(self.cahandler.profile_name)

    @patch.dict("os.environ", {"api_key_var": "key_var"})
    def test_063_config_key_load(self):
        """test _config_load - load template with key variable"""
        config_dic = {"api_key_variable": "api_key_var", "api_key": "api_key"}
        self.cahandler._config_key_load(config_dic)
        # with self.assertLogs('test_a2c', level='INFO') as lcm:
        self.assertEqual("api_key", self.cahandler.api_key)
        # self.assertIn("foo", lcm.output)
        self.assertFalse(self.cahandler.profile_name)

    @patch.dict("os.environ", {"api_password_var": "password_var"})
    def test_064_config_password_load(self):
        """test _config_load - load template with password variable"""
        config_dic = {"api_password_variable": "api_password_var"}
        self.cahandler._config_password_load(config_dic)
        self.assertEqual("password_var", self.cahandler.api_password)
        self.assertFalse(self.cahandler.profile_name)

    @patch.dict("os.environ", {"api_password_var": "password_var"})
    def test_065_config_password_load(self):
        """test _config_load - load template with password variable"""
        config_dic = {"api_password_variable": "does_not_exist"}
        with self.assertLogs("test_a2c", level="INFO") as lcm:
            self.cahandler._config_password_load(config_dic)
        self.assertFalse(self.cahandler.api_password)
        self.assertIn(
            "ERROR:test_a2c:Could not load password_variable: does_not_exist",
            lcm.output,
        )
        self.assertFalse(self.cahandler.profile_name)

    @patch.dict("os.environ", {"api_password_var": "password_var"})
    def test_066_config_password_load(self):
        """test _config_load - load template with password variable"""
        config_dic = {
            "api_password_variable": "api_password_var",
            "api_password": "api_password",
        }
        self.cahandler._config_password_load(config_dic)
        # with self.assertLogs('test_a2c', level='INFO') as lcm:
        self.assertEqual("api_password", self.cahandler.api_password)
        # self.assertIn("foo", lcm.output)
        self.assertFalse(self.cahandler.profile_name)

    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._validity_dates_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.CAhandler._csr_cn_get")
    @patch("acme2certifier.cahandlers.asa_ca_handler.csr_pubkey_get")
    def test_067_enrollment_dic_create(self, mock_pkg, mock_ccg, mock_vdg):
        """test _enrollment_dic_create()"""
        mock_pkg.return_value = "pubkey"
        mock_ccg.return_value = "cn"
        mock_vdg.return_value = ("date1", "date2")
        result = {
            "publicKey": "pubkey",
            "profileName": None,
            "issuerName": None,
            "cn": "cn",
            "notBefore": "date1",
            "notAfter": "date2",
        }
        self.assertEqual(result, self.cahandler._enrollment_dic_create("csr"))

    @patch("acme2certifier.cahandlers.asa_ca_handler.handler_config_check")
    def test_068_handler_check(self, mock_handler_check):
        """test handler_check"""
        mock_handler_check.return_value = "mock_handler_check"
        self.assertEqual("mock_handler_check", self.cahandler.handler_check())


if __name__ == "__main__":

    unittest.main()
